(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('akita-ng-fire'), require('rxjs/operators'), require('@angular/fire/database')) :
    typeof define === 'function' && define.amd ? define('akita-ng-fire/rtdb', ['exports', '@angular/core', 'akita-ng-fire', 'rxjs/operators', '@angular/fire/database'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global["akita-ng-fire"] = global["akita-ng-fire"] || {}, global["akita-ng-fire"].rtdb = {}), global.ng.core, global["akita-ng-fire"], global.rxjs.operators, global.ng.fire.database));
})(this, (function (exports, core, akitaNgFire, operators, database) { 'use strict';

    /** Set the configuration for the collection service */
    function RealTimeConfig(options) {
        if (options === void 0) { options = {}; }
        return function (constructor) {
            Object.keys(options).forEach(function (key) { return (constructor[key] = options[key]); });
        };
    }

    var RealTimeService = /** @class */ (function () {
        function RealTimeService(store, path, rtdb) {
            this.store = store;
            try {
                this.rtdb = rtdb || core.inject(database.Database);
            }
            catch (err) {
                throw new Error('RealTimeService requires "Database" provider.');
            }
            this.nodePath = path;
            this.pathRef = database.ref(this.rtdb, this.path);
        }
        Object.defineProperty(RealTimeService.prototype, "path", {
            get: function () {
                return this.constructor['nodeName'] || this.nodePath;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(RealTimeService.prototype, "idKey", {
            get: function () {
                return this.store ? this.store.idKey : 'id';
            },
            enumerable: false,
            configurable: true
        });
        /**
         * Function triggered when adding/updating data to the database
         * @note should be overridden
         */
        RealTimeService.prototype.formatToDatabase = function (entity) {
            return entity;
        };
        /**
         * Function triggered when getting data from database
         * @note should be overridden
         */
        RealTimeService.prototype.formatFromDatabase = function (entity) {
            return entity;
        };
        /**
         * @description just sync with node you specified when initialized the service class without updating the store
         */
        RealTimeService.prototype.syncNode = function () {
            var _this = this;
            return database.listVal(this.pathRef)
                .pipe(operators.map(function (value) { return _this.formatFromDatabase(value); }));
        };
        /**
         * @description sync the node with the store. `formatFromDatabase` will be called every time there is data incoming.
         */
        RealTimeService.prototype.syncNodeWithStore = function () {
            var _this = this;
            return database.stateChanges(this.pathRef).pipe(operators.tap(function (change) {
                var snapshot = change.snapshot, event = change.event;
                switch (event) {
                    case database.ListenEvent.added:
                        akitaNgFire.upsertStoreEntity(_this.store.storeName, _this.formatFromDatabase(snapshot.toJSON()), snapshot.key);
                        break;
                    case database.ListenEvent.removed:
                        akitaNgFire.removeStoreEntity(_this.store.storeName, snapshot.key);
                        break;
                    case database.ListenEvent.changed:
                        akitaNgFire.upsertStoreEntity(_this.store.storeName, _this.formatFromDatabase(snapshot.toJSON()), snapshot.key);
                        break;
                }
            }));
        };
        /**
         * @description adds a store entity to your database.
         * @param entity to add.
         */
        RealTimeService.prototype.add = function (entity) {
            var _a;
            var _this = this;
            if (entity[this.idKey]) {
                var entityRef = database.ref(this.rtdb, this.nodePath + '/' + entity[this.idKey]);
                return database.set(entityRef, this.formatToDatabase(entity));
            }
            if (Array.isArray(entity)) {
                var ids_1 = [];
                var promises = entity.map(function (e) {
                    var _a;
                    var pushRef = database.push(_this.pathRef);
                    var id = pushRef.key;
                    ids_1.push(id);
                    return database.set(pushRef, _this.formatToDatabase(Object.assign(Object.assign({}, e), (_a = {}, _a[_this.idKey] = id, _a))));
                });
                return Promise.all(promises).then(function () { return ids_1; });
            }
            else {
                var pushRef = database.push(this.pathRef);
                var id_1 = pushRef.key;
                return database.set(pushRef, this.formatToDatabase(Object.assign(Object.assign({}, entity), (_a = {}, _a[this.idKey] = id_1, _a))))
                    .then(function () { return id_1; });
            }
        };
        RealTimeService.prototype.update = function (idOrEntity, entity) {
            var _a, _b;
            var _this = this;
            if (Array.isArray(idOrEntity)) {
                return Promise.all(idOrEntity.map(function (e) {
                    var _a;
                    return database.update(_this.pathRef, (_a = {}, _a[e[_this.idKey]] = _this.formatToDatabase(e), _a));
                }));
            }
            else {
                if (typeof idOrEntity === 'string') {
                    return database.update(this.pathRef, (_a = {}, _a[idOrEntity] = this.formatToDatabase(entity), _a));
                }
                else if (typeof idOrEntity === 'object') {
                    var id = idOrEntity[this.idKey];
                    return database.update(this.pathRef, (_b = {}, _b[id] = this.formatToDatabase(idOrEntity), _b));
                }
                else {
                    return new Error("Couldn't find corresponding entity/ies: " + idOrEntity + ", " + entity);
                }
            }
        };
        /**
         *
         * @param id of the entity to remove
         */
        RealTimeService.prototype.remove = function (id) {
            try {
                var pathRef = database.ref(this.rtdb, this.path + "/" + id);
                return database.remove(pathRef);
            }
            catch (error) {
                return new Error("Error while removing entity with this id: " + id + ". " + error);
            }
        };
        /**
         * @warn clears the node and every child of the node.
         */
        RealTimeService.prototype.clearNode = function () {
            return database.remove(database.ref(this.rtdb, this.nodePath));
        };
        return RealTimeService;
    }());

    /**
     * Generated bundle index. Do not edit.
     */

    exports.RealTimeConfig = RealTimeConfig;
    exports.RealTimeService = RealTimeService;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=akita-ng-fire-rtdb.umd.js.map
